@@ -8,11 +8,10 @@ Tests for samples on README and RFCs related to CWT/COSE.
 """
 from secrets import token_bytes
 
-import cbor2
 import pytest
 
 import cwt
-from cwt import Claims, COSEKey, EncryptedCOSEKey
+from cwt import CWT, Claims, COSEKey, EncryptedCOSEKey, load_pem_hcert_dsc
 
 from .utils import key_path, now
 
@@ -80,7 +79,7 @@ class TestSample:
     """
 
     def test_sample_readme_maced_cwt_with_json_dict_old(self):
-        key = COSEKey.from_symmetric_key("mysecretpassword", alg="HS256")
+        key = COSEKey.from_symmetric_key("mysecretpassword", alg="HS256", kid="01")
         encoded = cwt.encode_and_mac(
             Claims.from_json(
                 {"iss": "coaps://as.example", "sub": "dajiaji", "cti": "123"}
@@ -96,7 +95,7 @@ class TestSample:
         assert 7 in decoded and decoded[7] == b"123"
 
     def test_sample_readme_maced_cwt_with_json_dict(self):
-        key = COSEKey.from_symmetric_key(alg="HMAC 256/256")
+        key = COSEKey.from_symmetric_key(alg="HMAC 256/256", kid="01")
         token = cwt.encode(
             {"iss": "coaps://as.example", "sub": "dajiaji", "cti": "123"}, key
         )
@@ -104,7 +103,7 @@ class TestSample:
         assert 1 in decoded and decoded[1] == "coaps://as.example"
 
     def test_sample_readme_maced_cwt_with_json_str_old(self):
-        key = COSEKey.from_symmetric_key("mysecretpassword", alg="HS256")
+        key = COSEKey.from_symmetric_key("mysecretpassword", alg="HS256", kid="01")
         encoded = cwt.encode_and_mac(
             Claims.from_json(
                 '{"iss":"coaps://as.example","sub":"dajiaji","cti":"123"}'
@@ -115,7 +114,7 @@ class TestSample:
         assert 1 in decoded and decoded[1] == "coaps://as.example"
 
     def test_sample_readme_maced_cwt_with_json_str(self):
-        key = COSEKey.from_symmetric_key(alg="HMAC 256/256")
+        key = COSEKey.from_symmetric_key(alg="HMAC 256/256", kid="01")
         token = cwt.encode(
             '{"iss":"coaps://as.example","sub":"dajiaji","cti":"123"}', key
         )
@@ -123,7 +122,7 @@ class TestSample:
         assert 1 in decoded and decoded[1] == "coaps://as.example"
 
     def test_sample_readme_maced_cwt_with_json_bytes_old(self):
-        key = COSEKey.from_symmetric_key("mysecretpassword", alg="HS256")
+        key = COSEKey.from_symmetric_key("mysecretpassword", alg="HS256", kid="01")
         encoded = cwt.encode_and_mac(
             Claims.from_json(
                 b'{"iss":"coaps://as.example","sub":"dajiaji","cti":"123"}'
@@ -134,7 +133,7 @@ class TestSample:
         assert 1 in decoded and decoded[1] == "coaps://as.example"
 
     def test_sample_readme_maced_cwt_with_json_bytes(self):
-        key = COSEKey.from_symmetric_key(alg="HMAC 256/256")
+        key = COSEKey.from_symmetric_key(alg="HMAC 256/256", kid="01")
         token = cwt.encode(
             b'{"iss":"coaps://as.example","sub":"dajiaji","cti":"123"}', key
         )
@@ -142,7 +141,7 @@ class TestSample:
         assert 1 in decoded and decoded[1] == "coaps://as.example"
 
     def test_sample_readme_maced_cwt_old(self):
-        key = COSEKey.from_symmetric_key("mysecretpassword", alg="HS256")
+        key = COSEKey.from_symmetric_key("mysecretpassword", alg="HS256", kid="01")
         encoded = cwt.encode_and_mac(
             {1: "coaps://as.example", 2: "dajiaji", 7: b"123"}, key
         )
@@ -150,16 +149,16 @@ class TestSample:
         assert 1 in decoded and decoded[1] == "coaps://as.example"
 
     def test_sample_readme_maced_cwt(self):
-        key = COSEKey.from_symmetric_key(alg="HMAC 256/256")
+        key = COSEKey.from_symmetric_key(alg="HMAC 256/256", kid="01")
         token = cwt.encode({1: "coaps://as.example", 2: "dajiaji", 7: b"123"}, key)
         decoded = cwt.decode(token, key)
         assert 1 in decoded and decoded[1] == "coaps://as.example"
 
     def test_sample_readme_signed_cwt_es256_old(self):
         with open(key_path("private_key_es256.pem")) as key_file:
-            private_key = COSEKey.from_pem(key_file.read())
+            private_key = COSEKey.from_pem(key_file.read(), kid="01")
         with open(key_path("public_key_es256.pem")) as key_file:
-            public_key = COSEKey.from_pem(key_file.read())
+            public_key = COSEKey.from_pem(key_file.read(), kid="01")
 
         encoded = cwt.encode_and_sign(
             Claims.from_json(
@@ -173,9 +172,9 @@ class TestSample:
 
     def test_sample_readme_signed_cwt_es256(self):
         with open(key_path("private_key_es256.pem")) as key_file:
-            private_key = COSEKey.from_pem(key_file.read())
+            private_key = COSEKey.from_pem(key_file.read(), kid="01")
         with open(key_path("public_key_es256.pem")) as key_file:
-            public_key = COSEKey.from_pem(key_file.read())
+            public_key = COSEKey.from_pem(key_file.read(), kid="01")
 
         token = cwt.encode(
             {"iss": "coaps://as.example", "sub": "dajiaji", "cti": "123"}, private_key
@@ -186,9 +185,9 @@ class TestSample:
 
     def test_sample_readme_signed_cwt_es384_old(self):
         with open(key_path("private_key_es384.pem")) as key_file:
-            private_key = COSEKey.from_pem(key_file.read())
+            private_key = COSEKey.from_pem(key_file.read(), kid="01")
         with open(key_path("public_key_es384.pem")) as key_file:
-            public_key = COSEKey.from_pem(key_file.read())
+            public_key = COSEKey.from_pem(key_file.read(), kid="01")
 
         encoded = cwt.encode_and_sign(
             Claims.from_json(
@@ -210,23 +209,19 @@ class TestSample:
 
     def test_sample_readme_signed_cwt_es384(self):
         with open(key_path("private_key_es384.pem")) as key_file:
-            private_key = COSEKey.from_pem(key_file.read())
-        with open(key_path("public_key_es384.pem")) as key_file:
-            public_key = COSEKey.from_pem(key_file.read())
-
+            private_key = COSEKey.from_pem(key_file.read(), kid="01")
         token = cwt.encode(
             {"iss": "coaps://as.example", "sub": "dajiaji", "cti": "123"}, private_key
         )
 
+        with open(key_path("public_key_es384.pem")) as key_file:
+            public_key = COSEKey.from_pem(key_file.read(), kid="01")
         decoded = cwt.decode(token, public_key)
         assert 1 in decoded and decoded[1] == "coaps://as.example"
 
     def test_sample_readme_signed_cwt_es512_old(self):
         with open(key_path("private_key_es512.pem")) as key_file:
-            private_key = COSEKey.from_pem(key_file.read())
-        with open(key_path("public_key_es512.pem")) as key_file:
-            public_key = COSEKey.from_pem(key_file.read())
-
+            private_key = COSEKey.from_pem(key_file.read(), kid="01")
         encoded = cwt.encode_and_sign(
             Claims.from_json(
                 {"iss": "coaps://as.example", "sub": "dajiaji", "cti": "123"}
@@ -234,28 +229,26 @@ class TestSample:
             private_key,
         )
 
+        with open(key_path("public_key_es512.pem")) as key_file:
+            public_key = COSEKey.from_pem(key_file.read(), kid="01")
         decoded = cwt.decode(encoded, public_key)
         assert 1 in decoded and decoded[1] == "coaps://as.example"
 
     def test_sample_readme_signed_cwt_es512(self):
         with open(key_path("private_key_es512.pem")) as key_file:
-            private_key = COSEKey.from_pem(key_file.read())
-        with open(key_path("public_key_es512.pem")) as key_file:
-            public_key = COSEKey.from_pem(key_file.read())
-
+            private_key = COSEKey.from_pem(key_file.read(), kid="01")
         token = cwt.encode(
             {"iss": "coaps://as.example", "sub": "dajiaji", "cti": "123"}, private_key
         )
 
+        with open(key_path("public_key_es512.pem")) as key_file:
+            public_key = COSEKey.from_pem(key_file.read(), kid="01")
         decoded = cwt.decode(token, public_key)
         assert 1 in decoded and decoded[1] == "coaps://as.example"
 
     def test_sample_readme_signed_cwt_es256k_old(self):
         with open(key_path("private_key_es256k.pem")) as key_file:
-            private_key = COSEKey.from_pem(key_file.read())
-        with open(key_path("public_key_es256k.pem")) as key_file:
-            public_key = COSEKey.from_pem(key_file.read())
-
+            private_key = COSEKey.from_pem(key_file.read(), kid="01")
         encoded = cwt.encode_and_sign(
             Claims.from_json(
                 {"iss": "coaps://as.example", "sub": "dajiaji", "cti": "123"}
@@ -263,28 +256,26 @@ class TestSample:
             private_key,
         )
 
+        with open(key_path("public_key_es256k.pem")) as key_file:
+            public_key = COSEKey.from_pem(key_file.read(), kid="01")
         decoded = cwt.decode(encoded, public_key)
         assert 1 in decoded and decoded[1] == "coaps://as.example"
 
     def test_sample_readme_signed_cwt_es256k(self):
         with open(key_path("private_key_es256k.pem")) as key_file:
-            private_key = COSEKey.from_pem(key_file.read())
-        with open(key_path("public_key_es256k.pem")) as key_file:
-            public_key = COSEKey.from_pem(key_file.read())
-
+            private_key = COSEKey.from_pem(key_file.read(), kid="01")
         token = cwt.encode(
             {"iss": "coaps://as.example", "sub": "dajiaji", "cti": "123"}, private_key
         )
 
+        with open(key_path("public_key_es256k.pem")) as key_file:
+            public_key = COSEKey.from_pem(key_file.read(), kid="01")
         decoded = cwt.decode(token, public_key)
         assert 1 in decoded and decoded[1] == "coaps://as.example"
 
     def test_sample_readme_signed_cwt_ed25519_old(self):
         with open(key_path("private_key_ed25519.pem")) as key_file:
-            private_key = COSEKey.from_pem(key_file.read())
-        with open(key_path("public_key_ed25519.pem")) as key_file:
-            public_key = COSEKey.from_pem(key_file.read())
-
+            private_key = COSEKey.from_pem(key_file.read(), kid="01")
         encoded = cwt.encode_and_sign(
             Claims.from_json(
                 {"iss": "coaps://as.example", "sub": "dajiaji", "cti": "123"}
@@ -292,52 +283,84 @@ class TestSample:
             private_key,
         )
 
+        with open(key_path("public_key_ed25519.pem")) as key_file:
+            public_key = COSEKey.from_pem(key_file.read(), kid="01")
         decoded = cwt.decode(encoded, public_key)
         assert 1 in decoded and decoded[1] == "coaps://as.example"
 
     def test_sample_readme_signed_cwt_ed25519(self):
         with open(key_path("private_key_ed25519.pem")) as key_file:
-            private_key = COSEKey.from_pem(key_file.read())
-        with open(key_path("public_key_ed25519.pem")) as key_file:
-            public_key = COSEKey.from_pem(key_file.read())
-
+            private_key = COSEKey.from_pem(key_file.read(), kid="01")
         token = cwt.encode(
             {"iss": "coaps://as.example", "sub": "dajiaji", "cti": "123"}, private_key
         )
 
+        with open(key_path("public_key_ed25519.pem")) as key_file:
+            public_key = COSEKey.from_pem(key_file.read(), kid="01")
+        decoded = cwt.decode(token, public_key)
+        assert 1 in decoded and decoded[1] == "coaps://as.example"
+
+    def test_sample_readme_signed_cwt_ed25519_with_jwk(self):
+
+        # The sender side:
+        private_key = COSEKey.from_jwk(
+            {
+                "kid": "01",
+                "kty": "OKP",
+                "key_ops": ["sign"],
+                "alg": "EdDSA",
+                "crv": "Ed25519",
+                "x": "2E6dX83gqD_D0eAmqnaHe1TC1xuld6iAKXfw2OVATr0",
+                "d": "L8JS08VsFZoZxGa9JvzYmCWOwg7zaKcei3KZmYsj7dc",
+            }
+        )
+        token = cwt.encode(
+            {"iss": "coaps://as.example", "sub": "dajiaji", "cti": "123"}, private_key
+        )
+
+        # The recipient side:
+        public_key = COSEKey.from_jwk(
+            {
+                "kid": "01",
+                "kty": "OKP",
+                "key_ops": ["verify"],
+                "crv": "Ed25519",
+                "x": "2E6dX83gqD_D0eAmqnaHe1TC1xuld6iAKXfw2OVATr0",
+            }
+        )
         decoded = cwt.decode(token, public_key)
         assert 1 in decoded and decoded[1] == "coaps://as.example"
 
     def test_sample_readme_signed_cwt_rs256(self):
         with open(key_path("private_key_rsa.pem")) as key_file:
-            private_key = COSEKey.from_pem(key_file.read(), alg="RS256")
-        with open(key_path("public_key_rsa.pem")) as key_file:
-            public_key = COSEKey.from_pem(key_file.read(), alg="RS256")
-
+            private_key = COSEKey.from_pem(key_file.read(), alg="RS256", kid="01")
         token = cwt.encode(
             {"iss": "coaps://as.example", "sub": "dajiaji", "cti": "123"}, private_key
         )
 
+        with open(key_path("public_key_rsa.pem")) as key_file:
+            public_key = COSEKey.from_pem(key_file.read(), alg="RS256", kid="01")
         decoded = cwt.decode(token, public_key)
         assert 1 in decoded and decoded[1] == "coaps://as.example"
 
     def test_sample_readme_signed_cwt_ps256(self):
         with open(key_path("private_key_rsa.pem")) as key_file:
-            private_key = COSEKey.from_pem(key_file.read(), alg="PS256")
-        with open(key_path("public_key_rsa.pem")) as key_file:
-            public_key = COSEKey.from_pem(key_file.read(), alg="PS256")
-
+            private_key = COSEKey.from_pem(key_file.read(), alg="PS256", kid="01")
         token = cwt.encode(
             {"iss": "coaps://as.example", "sub": "dajiaji", "cti": "123"}, private_key
         )
 
+        with open(key_path("public_key_rsa.pem")) as key_file:
+            public_key = COSEKey.from_pem(key_file.read(), alg="PS256", kid="01")
         decoded = cwt.decode(token, public_key)
         assert 1 in decoded and decoded[1] == "coaps://as.example"
 
     def test_sample_readme_encrypted_cwt_old(self):
         nonce = token_bytes(13)
         mysecret = token_bytes(32)
-        enc_key = COSEKey.from_symmetric_key(mysecret, alg="AES-CCM-16-64-256")
+        enc_key = COSEKey.from_symmetric_key(
+            mysecret, alg="AES-CCM-16-64-256", kid="01"
+        )
         encoded = cwt.encode_and_encrypt(
             Claims.from_json(
                 {"iss": "coaps://as.example", "sub": "dajiaji", "cti": "123"}
@@ -349,7 +372,7 @@ class TestSample:
         assert 1 in decoded and decoded[1] == "coaps://as.example"
 
     def test_sample_readme_encrypted_cwt(self):
-        enc_key = COSEKey.from_symmetric_key(alg="ChaCha20/Poly1305")
+        enc_key = COSEKey.from_symmetric_key(alg="ChaCha20/Poly1305", kid="01")
         token = cwt.encode(
             {"iss": "coaps://as.example", "sub": "dajiaji", "cti": "123"}, enc_key
         )
@@ -359,16 +382,12 @@ class TestSample:
     def test_sample_readme_nested_cwt_old(self):
         with open(key_path("private_key_es256.pem")) as key_file:
             private_key = COSEKey.from_pem(key_file.read(), kid="01")
-        with open(key_path("public_key_es256.pem")) as key_file:
-            public_key = COSEKey.from_pem(key_file.read(), kid="01")
-
         encoded = cwt.encode_and_sign(
             Claims.from_json(
                 {"iss": "coaps://as.example", "sub": "dajiaji", "cti": "123"}
             ),
             private_key,
         )
-
         nonce = token_bytes(13)
         mysecret = token_bytes(32)
         enc_key = COSEKey.from_symmetric_key(
@@ -376,46 +395,48 @@ class TestSample:
         )
         nested = cwt.encode_and_encrypt(encoded, enc_key, nonce=nonce)
 
+        with open(key_path("public_key_es256.pem")) as key_file:
+            public_key = COSEKey.from_pem(key_file.read(), kid="01")
         decoded = cwt.decode(nested, [enc_key, public_key])
         assert 1 in decoded and decoded[1] == "coaps://as.example"
 
     def test_sample_readme_nested_cwt(self):
         with open(key_path("private_key_es256.pem")) as key_file:
             private_key = COSEKey.from_pem(key_file.read(), kid="01")
-        with open(key_path("public_key_es256.pem")) as key_file:
-            public_key = COSEKey.from_pem(key_file.read(), kid="01")
-
         token = cwt.encode(
             {"iss": "coaps://as.example", "sub": "dajiaji", "cti": "123"}, private_key
         )
-
         enc_key = COSEKey.from_symmetric_key(alg="ChaCha20/Poly1305", kid="02")
         nested = cwt.encode(token, enc_key)
 
+        with open(key_path("public_key_es256.pem")) as key_file:
+            public_key = COSEKey.from_pem(key_file.read(), kid="01")
         decoded = cwt.decode(nested, [enc_key, public_key])
         assert 1 in decoded and decoded[1] == "coaps://as.example"
 
     def test_sample_readme_nested_cwt_without_kid(self):
+        enc_key = COSEKey.from_symmetric_key(alg="ChaCha20/Poly1305")
+
         with open(key_path("private_key_es256.pem")) as key_file:
             private_key = COSEKey.from_pem(key_file.read())
+        ctx = CWT.new()
+        ctx.cose.verify_kid = False
+        token = ctx.encode(
+            {"iss": "coaps://as.example", "sub": "dajiaji", "cti": "123"}, private_key
+        )
+        nested = ctx.encode(token, enc_key)
+
         with open(key_path("public_key_es256.pem")) as key_file:
             public_key = COSEKey.from_pem(key_file.read())
 
-        token = cwt.encode(
-            {"iss": "coaps://as.example", "sub": "dajiaji", "cti": "123"}, private_key
-        )
-
-        enc_key = COSEKey.from_symmetric_key(alg="ChaCha20/Poly1305")
-        nested = cwt.encode(token, enc_key)
-
-        decoded = cwt.decode(nested, [enc_key, public_key])
+        decoded = ctx.decode(nested, [enc_key, public_key])
         assert 1 in decoded and decoded[1] == "coaps://as.example"
 
     def test_sample_readme_cwt_with_pop_jwk(self):
 
         # issuer:
         with open(key_path("private_key_ed25519.pem")) as key_file:
-            private_key = COSEKey.from_pem(key_file.read())
+            private_key = COSEKey.from_pem(key_file.read(), kid="issuer-01")
         token = cwt.encode(
             {
                 "iss": "coaps://as.example",
@@ -452,7 +473,7 @@ class TestSample:
 
         # recipient:
         with open(key_path("public_key_ed25519.pem")) as key_file:
-            public_key = COSEKey.from_pem(key_file.read())
+            public_key = COSEKey.from_pem(key_file.read(), kid="issuer-01")
         decoded = cwt.decode(token, public_key)
         assert 8 in decoded and isinstance(decoded[8], dict)
         assert 1 in decoded[8] and isinstance(decoded[8][1], dict)
@@ -465,10 +486,11 @@ class TestSample:
 
     def test_sample_readme_cwt_with_pop_encrypted_cose_key_readable(self):
         with open(key_path("private_key_ed25519.pem")) as key_file:
-            private_key = COSEKey.from_pem(key_file.read())
+            private_key = COSEKey.from_pem(key_file.read(), kid="issuer-01")
         enc_key = COSEKey.from_symmetric_key(
             "a-client-secret-of-cwt-recipient",  # Just 32 bytes!
             alg="ChaCha20/Poly1305",
+            kid="presenter-01",
         )
         pop_key = COSEKey.from_symmetric_key(
             "a-client-secret-of-cwt-presenter",
@@ -488,7 +510,7 @@ class TestSample:
         )
 
         with open(key_path("public_key_ed25519.pem")) as key_file:
-            public_key = COSEKey.from_pem(key_file.read())
+            public_key = COSEKey.from_pem(key_file.read(), kid="issuer-01")
         decoded = cwt.decode(token, public_key)
         assert 8 in decoded and isinstance(decoded[8], dict)
         assert 2 in decoded[8] and isinstance(decoded[8][2], list)
@@ -500,7 +522,7 @@ class TestSample:
 
     def test_sample_readme_cwt_with_pop_kid_readable(self):
         with open(key_path("private_key_ed25519.pem")) as key_file:
-            private_key = COSEKey.from_pem(key_file.read())
+            private_key = COSEKey.from_pem(key_file.read(), kid="issuer-01")
 
         token = cwt.encode(
             {
@@ -515,7 +537,7 @@ class TestSample:
         )
 
         with open(key_path("public_key_ed25519.pem")) as key_file:
-            public_key = COSEKey.from_pem(key_file.read())
+            public_key = COSEKey.from_pem(key_file.read(), kid="issuer-01")
         decoded = cwt.decode(token, public_key)
         assert 8 in decoded and isinstance(decoded[8], dict)
         assert 3 in decoded[8] and decoded[8][3] == b"pop-key-id-of-cwt-presenter"
@@ -524,7 +546,7 @@ class TestSample:
 
     def test_sample_readme_cwt_with_pop_cose_key(self):
         with open(key_path("private_key_ed25519.pem")) as key_file:
-            private_key = COSEKey.from_pem(key_file.read())
+            private_key = COSEKey.from_pem(key_file.read(), kid="issuer-01")
         with open(key_path("public_key_es256.pem")) as key_file:
             pop_key = COSEKey.from_pem(key_file.read())
         token = cwt.encode(
@@ -540,7 +562,7 @@ class TestSample:
         )
 
         with open(key_path("public_key_ed25519.pem")) as key_file:
-            public_key = COSEKey.from_pem(key_file.read())
+            public_key = COSEKey.from_pem(key_file.read(), kid="issuer-01")
         decoded = cwt.decode(token, public_key)
         assert 8 in decoded and isinstance(decoded[8], dict)
         assert 1 in decoded[8] and isinstance(decoded[8][1], dict)
@@ -550,10 +572,11 @@ class TestSample:
 
     def test_sample_readme_cwt_with_pop_encrypted_cose_key(self):
         with open(key_path("private_key_ed25519.pem")) as key_file:
-            private_key = COSEKey.from_pem(key_file.read())
+            private_key = COSEKey.from_pem(key_file.read(), kid="issuer-01")
         enc_key = COSEKey.from_symmetric_key(
             "a-client-secret-of-cwt-recipient",  # Just 32 bytes!
             alg="ChaCha20/Poly1305",
+            kid="presenter-01",
         )
         pop_key = COSEKey.from_symmetric_key(
             "a-client-secret-of-cwt-presenter",
@@ -572,7 +595,7 @@ class TestSample:
         )
 
         with open(key_path("public_key_ed25519.pem")) as key_file:
-            public_key = COSEKey.from_pem(key_file.read())
+            public_key = COSEKey.from_pem(key_file.read(), kid="issuer-01")
         decoded = cwt.decode(token, public_key)
         assert 8 in decoded and isinstance(decoded[8], dict)
         assert 2 in decoded[8] and isinstance(decoded[8][2], list)
@@ -583,7 +606,7 @@ class TestSample:
 
     def test_sample_readme_cwt_with_pop_kid(self):
         with open(key_path("private_key_ed25519.pem")) as key_file:
-            private_key = COSEKey.from_pem(key_file.read())
+            private_key = COSEKey.from_pem(key_file.read(), kid="01")
         token = cwt.encode(
             {
                 1: "coaps://as.example",  # iss
@@ -597,16 +620,14 @@ class TestSample:
         )
 
         with open(key_path("public_key_ed25519.pem")) as key_file:
-            public_key = COSEKey.from_pem(key_file.read())
+            public_key = COSEKey.from_pem(key_file.read(), kid="01")
         decoded = cwt.decode(token, public_key)
         assert 8 in decoded and isinstance(decoded[8], dict)
         assert 3 in decoded[8] and decoded[8][3] == b"pop-key-id-of-cwt-presenter"
 
     def test_sample_readme_cwt_with_user_defined_claims(self):
         with open(key_path("private_key_ed25519.pem")) as key_file:
-            private_key = COSEKey.from_pem(key_file.read())
-        with open(key_path("public_key_ed25519.pem")) as key_file:
-            public_key = COSEKey.from_pem(key_file.read())
+            private_key = COSEKey.from_pem(key_file.read(), kid="01")
         token = cwt.encode(
             {
                 1: "coaps://as.example",  # iss
@@ -619,6 +640,9 @@ class TestSample:
             },
             private_key,
         )
+
+        with open(key_path("public_key_ed25519.pem")) as key_file:
+            public_key = COSEKey.from_pem(key_file.read(), kid="01")
         raw = cwt.decode(token, public_key)
         assert raw[-70001] == "foo"
         assert isinstance(raw[-70002], list)
@@ -634,9 +658,7 @@ class TestSample:
 
     def test_sample_readme_cwt_with_user_defined_claims_readable(self):
         with open(key_path("private_key_ed25519.pem")) as key_file:
-            private_key = COSEKey.from_pem(key_file.read())
-        with open(key_path("public_key_ed25519.pem")) as key_file:
-            public_key = COSEKey.from_pem(key_file.read())
+            private_key = COSEKey.from_pem(key_file.read(), kid="01")
         cwt.set_private_claim_names(
             {
                 "ext_1": -70001,
@@ -657,6 +679,9 @@ class TestSample:
             },
             private_key,
         )
+
+        with open(key_path("public_key_ed25519.pem")) as key_file:
+            public_key = COSEKey.from_pem(key_file.read(), kid="01")
         raw = cwt.decode(token, public_key)
         readable = Claims.new(
             raw,
@@ -673,10 +698,6 @@ class TestSample:
         assert readable.get("ext_4") == 123
 
     def test_sample_readme_decode_with_multiple_keys(self):
-        with open(key_path("public_key_es256.pem")) as key_file:
-            public_key_1 = COSEKey.from_pem(key_file.read(), kid="01")
-        with open(key_path("public_key_ed25519.pem")) as key_file:
-            public_key_2 = COSEKey.from_pem(key_file.read(), kid="02")
         with open(key_path("private_key_ed25519.pem")) as key_file:
             private_key = COSEKey.from_pem(key_file.read(), kid="02")
         token = cwt.encode(
@@ -687,6 +708,11 @@ class TestSample:
             },
             private_key,
         )
+
+        with open(key_path("public_key_es256.pem")) as key_file:
+            public_key_1 = COSEKey.from_pem(key_file.read(), kid="01")
+        with open(key_path("public_key_ed25519.pem")) as key_file:
+            public_key_2 = COSEKey.from_pem(key_file.read(), kid="02")
         decoded = cwt.decode(token, [public_key_1, public_key_2])
         assert 1 in decoded and decoded[1] == "coaps://as.example"
 
@@ -808,8 +834,6 @@ class TestSample:
             key=key,
             nonce=nonce,
         )
-        print(cbor2.loads(encoded))
-        print(cbor2.loads(bytes.fromhex(SAMPLE_CWT_RFC8392_A5)))
         assert encoded == bytes.fromhex(SAMPLE_CWT_RFC8392_A5)
         decoded = cwt.decode(encoded, keys=key, no_verify=True)
         assert 1 in decoded and decoded[1] == "coap://as.example.com"
@@ -880,3 +904,58 @@ class TestSample:
         encrypted = cwt.encode(signed, key=enc_key, nonce=nonce)
         decoded = cwt.decode(encrypted, keys=[enc_key, sig_key], no_verify=True)
         assert 1 in decoded and decoded[1] == "coap://as.example.com"
+
+    def test_sample_hcert_testdata_AT_2DCode_raw_1(self):
+
+        # A DSC(Document Signing Certificate) issued by a CSCA (Certificate Signing Certificate Authority).
+        dsc = "-----BEGIN CERTIFICATE-----\nMIIBvTCCAWOgAwIBAgIKAXk8i88OleLsuTAKBggqhkjOPQQDAjA2MRYwFAYDVQQDDA1BVCBER0MgQ1NDQSAxMQswCQYDVQQGEwJBVDEPMA0GA1UECgwGQk1TR1BLMB4XDTIxMDUwNTEyNDEwNloXDTIzMDUwNTEyNDEwNlowPTERMA8GA1UEAwwIQVQgRFNDIDExCzAJBgNVBAYTAkFUMQ8wDQYDVQQKDAZCTVNHUEsxCjAIBgNVBAUTATEwWTATBgcqhkjOPQIBBggqhkjOPQMBBwNCAASt1Vz1rRuW1HqObUE9MDe7RzIk1gq4XW5GTyHuHTj5cFEn2Rge37+hINfCZZcozpwQKdyaporPUP1TE7UWl0F3o1IwUDAOBgNVHQ8BAf8EBAMCB4AwHQYDVR0OBBYEFO49y1ISb6cvXshLcp8UUp9VoGLQMB8GA1UdIwQYMBaAFP7JKEOflGEvef2iMdtopsetwGGeMAoGCCqGSM49BAMCA0gAMEUCIQDG2opotWG8tJXN84ZZqT6wUBz9KF8D+z9NukYvnUEQ3QIgdBLFSTSiDt0UJaDF6St2bkUQuVHW6fQbONd731/M4nc=\n-----END CERTIFICATE-----"
+
+        # An EUDCC (EU Digital COVID Certificate)
+        eudcc = bytes.fromhex(
+            "d2844da20448d919375fc1e7b6b20126a0590133a4041a61817ca0061a60942ea001624154390103a101a4617681aa62646e01626d616d4f52472d3130303033303231356276706a313131393334393030376264746a323032312d30322d313862636f624154626369783155524e3a555643493a30313a41543a31303830373834334639344145453045453530393346424332353442443831332342626d706c45552f312f32302f31353238626973781b4d696e6973747279206f66204865616c74682c20417573747269616273640262746769383430353339303036636e616da463666e74754d5553544552465241553c474f455353494e47455262666e754d7573746572667261752d47c3b6c39f696e67657263676e74684741425249454c4562676e684761627269656c656376657265312e302e3063646f626a313939382d30322d323658405812fce67cb84c3911d78e3f61f890d0c80eb9675806aebed66aa2d0d0c91d1fc98d7bcb80bf00e181806a9502e11b071325901bd0d2c1b6438747b8cc50f521"
+        )
+
+        public_key = load_pem_hcert_dsc(dsc)
+        decoded = cwt.decode(eudcc, keys=[public_key], no_verify=True)
+        claims = Claims.new(decoded)
+
+        assert 1 in claims.hcert
+        assert isinstance(claims.hcert[1], dict)
+        assert "v" in claims.hcert[1]
+        assert "nam" in claims.hcert[1]
+        assert "dob" in claims.hcert[1]
+        assert "ver" in claims.hcert[1]
+        assert isinstance(claims.hcert[1]["v"], list)
+        assert len(claims.hcert[1]["v"]) == 1
+        assert isinstance(claims.hcert[1]["v"][0], dict)
+        assert isinstance(claims.hcert[1]["nam"], dict)
+        assert "fnt" in claims.hcert[1]["nam"]
+        assert claims.hcert[1]["nam"]["fnt"] == "MUSTERFRAU<GOESSINGER"
+        assert claims.hcert[1]["dob"] == "1998-02-26"
+        assert claims.hcert[1]["ver"] == "1.0.0"
+
+    def test_sample_hcert_testdata_AT_2DCode_raw_1_with_cert_file(self):
+        eudcc = bytes.fromhex(
+            "d2844da20448d919375fc1e7b6b20126a0590133a4041a61817ca0061a60942ea001624154390103a101a4617681aa62646e01626d616d4f52472d3130303033303231356276706a313131393334393030376264746a323032312d30322d313862636f624154626369783155524e3a555643493a30313a41543a31303830373834334639344145453045453530393346424332353442443831332342626d706c45552f312f32302f31353238626973781b4d696e6973747279206f66204865616c74682c20417573747269616273640262746769383430353339303036636e616da463666e74754d5553544552465241553c474f455353494e47455262666e754d7573746572667261752d47c3b6c39f696e67657263676e74684741425249454c4562676e684761627269656c656376657265312e302e3063646f626a313939382d30322d323658405812fce67cb84c3911d78e3f61f890d0c80eb9675806aebed66aa2d0d0c91d1fc98d7bcb80bf00e181806a9502e11b071325901bd0d2c1b6438747b8cc50f521"
+        )
+
+        with open(key_path("hcert_testdata_cert_at.pem")) as key_file:
+            dsc = key_file.read()
+            public_key = load_pem_hcert_dsc(dsc)
+        decoded = cwt.decode(eudcc, keys=[public_key], no_verify=True)
+        claims = Claims.new(decoded)
+
+        assert 1 in claims.hcert
+        assert isinstance(claims.hcert[1], dict)
+        assert "v" in claims.hcert[1]
+        assert "nam" in claims.hcert[1]
+        assert "dob" in claims.hcert[1]
+        assert "ver" in claims.hcert[1]
+        assert isinstance(claims.hcert[1]["v"], list)
+        assert len(claims.hcert[1]["v"]) == 1
+        assert isinstance(claims.hcert[1]["v"][0], dict)
+        assert isinstance(claims.hcert[1]["nam"], dict)
+        assert "fnt" in claims.hcert[1]["nam"]
+        assert claims.hcert[1]["nam"]["fnt"] == "MUSTERFRAU<GOESSINGER"
+        assert claims.hcert[1]["dob"] == "1998-02-26"
+        assert claims.hcert[1]["ver"] == "1.0.0"
